#include "QTPBoard.h"
#include <coecobs.h>
QTPBoard::QTPBoard(Tickypip *tickypip)
	: tickypip(tickypip)
{
	board = new Board();
	board->setUpdateListener(this);
	resetSizes();

	clear();
}

QTPBoard::~QTPBoard()
{
	if (board != NULL)
		delete board;
}

void
QTPBoard::loadLevel(TDes&filename, size_t num)
{
	getBoard()->loadLevel(filename, num);
	tickypip->reset();
	resetSizes();
}

void
QTPBoard::setBoard(Board *board)
{
	if (this->board != NULL)
		delete this->board;

	this->board = board;
}

Board *
QTPBoard::getBoard() const
{
	return board;
}

void
QTPBoard::clear()
{
	DrawNow();
}

void
QTPBoard::cellUpdated(int, int, Cell *cell)
{
	if(!iDisableUpdate)
	{
	ActivateGc();
	CWindowGc& gc= SystemGc();
	QPainter paint(&gc);
	drawCell(paint, cell);
	DeactivateGc();
	}
}

void
QTPBoard::resetSizes()
{
	int width  = board->getWidth()  * Cell::WIDTH  + 3;
	int height = board->getHeight() * Cell::HEIGHT + 3;
	SetSize(TSize(width,height));
	SetPosition(TPoint(tickypip->Size().iWidth/2-Size().iWidth/2,Position().iY));
}

void
QTPBoard::drawCell(QPainter &paint, Cell *cell)const
{
	if (cell == NULL)
		return;

	int tx = cell->getX() * Cell::WIDTH  + 1;
	int ty = cell->getY() * Cell::HEIGHT + 1;

	paint.translate(tx+Position().iX, ty+Position().iY);
	cell->paint(paint);
	paint.translate(-(tx+Position().iX), -(ty+Position().iY));
}

void
QTPBoard::drawCells(QPainter &paint)const
{
	unsigned int x, y;

	for (y = 0; y < board->getHeight(); y++)
	{
		for (x = 0; x < board->getWidth(); x++)
		{
			Cell *cell = board->getCell(x, y);

			if (cell == NULL)
			{
#if 0
				paint.fillRect(x * Cell::WIDTH + 1, y * Cell::HEIGHT + 1,
							   Cell::WIDTH, Cell::HEIGHT,
							   QColor(230, 230, 230));
#endif
			}
			else
				drawCell(paint, cell);
		}
	}
}

void
QTPBoard::drawBoard(QPainter &paint)const 
{
#if 0
	paint.setPen(Qt::black);
	paint.drawRect(0, 0, w + 2, h + 2);

	paint.setPen(QColor(164, 164, 164));
	paint.drawLine(1,     h + 2, w + 2, h + 2);
	paint.drawLine(w + 2, 1,     w + 2, h + 2);
#endif

	drawCells(paint);
}


void
QTPBoard::Draw(const TRect& aRect) const
{
	CWindowGc& gc = SystemGc();
	QPainter paint(&gc);
	drawBoard(paint);
}

void
QTPBoard::HandlePointerEventL(const TPointerEvent& aPointerEvent)
{
	if(aPointerEvent.iType ==TPointerEvent::EButton1Down)
	{
	if(!IsFocused())
	{
		ReportEventL(MCoeControlObserver::EEventRequestFocus);
	}
	CCoeControl::HandlePointerEventL(aPointerEvent);
	TPoint pos = aPointerEvent.iPosition-Position();
	int x = pos2index(pos.iX);
	int y = pos2index(pos.iY);
	Cell *cell = board->getCell(x, y);

	if (cell == NULL)
		return;

	if (cell->getType() == Cell::BLANK)
		return;

	cell->rotateLeft();

	tickypip->incMovesCount();

	/* Check if the level is completed. */
	if (board->isLevelCompleted())
		tickypip->levelCompleted();
	}
}

void
QTPBoard::PositionChanged()
{
			board->SetPos(Position());
}
