#include "TPLevel.h"
#include "BlankCell.h"
#include "ComputerCell.h"
#include "ConnectorCell.h"
#include "ServerCell.h"
#include <string.h>

const unsigned char MAGIC[14]="$TICKYPIP 0.1";

TPLevel::TPLevel(Board *board)
	: Level(board), topServerCell(NULL)
{
}

bool
TPLevel::isValidFormat()
{
	TBuf8<14>magic;
	TInt seek =0;
	f.Seek(ESeekStart,seek);
	f.Read(magic,13/* strlen(MAGIC) + 1*/);
	TPtrC8 ptr(MAGIC,13);
	if (magic != ptr)
		return false;

	return true;
}

const TDesC8& TPLevel::readLine()
{
	iBufLine.SetLength(0);
	TBuf8<1> theChar;
	TInt err=KErrNone;
	do
	{
		err=f.Read(theChar);
		iBufLine.Append(theChar);
	}while(theChar[0]!=10 && err==KErrNone);
	if(err == KErrNone) // ended with a new line
		iBufLine.SetLength(iBufLine.Length()-1);// skip\n 
	iLexer.Assign(iBufLine);
	return iBufLine;
}

bool
TPLevel::readHeader()
{
	TBuf8<80> numLevelStr;
	int numLevels;
	bool ok;

	readLine();
	readLine();
	TBuf<80> title;
	title.Copy(readLine());
	setTitle(title);

	numLevelStr = readLine();
	iLexer.Val(numLevels);//numLevelStr.toUInt(&ok);

	if (!ok)
		return false;

	setNumLevels(numLevels);

	return true;
}

bool
TPLevel::setupLevel(size_t level)
{
	if (getBoard() == NULL)
		return false;
	TInt seek=0;
	size_t width, height;
	int wrapped;
	size_t l;

	topServerCell = NULL;
	f.Seek(ESeekStart,seek);

	readLine();
	readLine();
	readLine();
	readLine();
	readLine();

	for (l = 0; l < level; l++)
	{
		if (!readLevelHeader(width, height, wrapped))
			return false;

		for (size_t i = 0; i <= height; i++)
			readLine();
	}

//	if (f.c())
//		return false;
#pragma message("Check size of file eof")
	getBoard()->clear();

	readLevelHeader(width, height, wrapped);

	getBoard()->setWidth(width);
	getBoard()->setHeight(height);
	getBoard()->setWrapped(wrapped);

	getBoard()->setupGrid();

	for (size_t y = 0; y < height; y++)
	{
		TBuf8<80>row;
		char cellOptions;
		char cellType;

		f.Read(row, width * 2 + 1);

		for (size_t j = 0, x = 0; x < width; j += 2, x++)
		{
			cellType    = row[j];
			cellOptions = row[j + 1];

			addCell(cellType, cellOptions, x, y);
		}
		seek = 0;
		f.Seek(ESeekCurrent,seek);
		if (seek>=iFileSize && y < height - 1)
			return false;
	}

	getBoard()->randomize();

	return true;
}

void
TPLevel::addCell(char cellType, char cellOptions, size_t x, size_t y)
{
	switch (cellType)
	{
		case CELL_BLANK:
			getBoard()->setCell(x, y, new BlankCell());
			break;

		case CELL_COMPUTER:
			getBoard()->setCell(x, y, new ComputerCell());
			getBoard()->setTotalComputers(getBoard()->getTotalComputers() + 1);
			break;

		case CELL_CONNECTOR:
		{
			ConnectorCell::ConnectorType type;

			switch (cellOptions)
			{
				case '0': type = ConnectorCell::CONNECTOR_LINE;  break;
				case '1': type = ConnectorCell::CONNECTOR_L;     break;
				case '2': type = ConnectorCell::CONNECTOR_T;     break;
				case '3': type = ConnectorCell::CONNECTOR_CROSS; break;
				case '4': type = ConnectorCell::CONNECTOR_END;   break;
				default:  type = ConnectorCell::CONNECTOR_END;   break;
			}

			if (topServerCell != NULL && topServerCell->getY() == y - 1 &&
				topServerCell->getX() == x)
			{
				ServerCell *cell = new ServerCell(ServerCell::SERVER_BOTTOM);
				getBoard()->setCell(x, y, cell);
				cell->setConnectorType(type);
				topServerCell->setConnectorType(type);
			}
			else
			{
				ConnectorCell *cell = new ConnectorCell();
				getBoard()->setCell(x, y, cell);
				cell->setConnectorType(type);
			}

			break;
		}

		case CELL_SERVER:
		{
			topServerCell = new ServerCell(ServerCell::SERVER_TOP);
			getBoard()->setCell(x, y, topServerCell);
			break;
		}

		default:
			getBoard()->setCell(x, y, NULL);
			break;
	}

	if (getBoard()->getCell(x, y) != NULL)
	{
		Cell *cell = getBoard()->getCell(x, y);
		cell->setX(x);
		cell->setY(y);
		cell->setBoard(getBoard());
	}
}

bool
TPLevel::readLevelHeader(size_t &width, size_t &height,
						 int & wrapped)
{
	bool ok;
	TLex8 lexer(readLine());
	TLex8 valLex;
	valLex.Assign(lexer.NextToken());
	ok = valLex.Val(width);
	if (ok!=KErrNone) return false;
	valLex.Assign(lexer.NextToken());
	ok=valLex.Val(height);
	if (ok!=KErrNone) return false;

	valLex.Assign(lexer.NextToken());
	ok=valLex.Val(wrapped);
	if (ok!=KErrNone) return false;

	return true;
}
